@@ -197,7 +197,7 @@ void server::handle_request(boost::asio::ip::tcp::socket& socket,
 	
 	foreach (const header& h, req.headers)
 	{
-		if (m_log_forwarded and h.name == "X-Forwarded-For")
+		if (m_log_forwarded and ba::iequals(h.name, "X-Forwarded-For"))
 		{
 			client = h.value;
 			string::size_type comma = client.rfind(',');
@@ -208,11 +208,11 @@ void server::handle_request(boost::asio::ip::tcp::socket& socket,
 				client = client.substr(comma + 1, string::npos);
 			}
 		}
-		else if (h.name == "Referer")
+		else if (ba::iequals(h.name, "Referer"))
 			referer = h.value;
-		else if (h.name == "User-Agent")
+		else if (ba::iequals(h.name, "User-Agent"))
 			userAgent = h.value;
-		else if (h.name == "Accept")
+		else if (ba::iequals(h.name, "Accept"))
 			accept = h.value;
 	}
 	
@@ -236,6 +236,10 @@ void server::handle_request(boost::asio::ip::tcp::socket& socket,
 			rep.set_content_type("text/html; charset=utf-8");
 		}
 	}
+	catch (exception& e)
+	{
+		rep = reply::stock_reply(internal_server_error, e.what());
+	}
 	catch (...)
 	{
 		rep = reply::stock_reply(internal_server_error);
@@ -255,15 +259,19 @@ void server::log_request(const string& client,
 	const string& entry)
 {
 	// protect the output stream from garbled log messages
-	boost::mutex::scoped_lock lock(detail::s_log_lock);
+	boost::unique_lock<boost::mutex> lock(detail::s_log_lock);
 
 	using namespace boost::local_time;
 
 	local_date_time start_local(start, time_zone_ptr());
 
+	string username = req.username;
+	if (username.empty())
+		username = "-";
+
 	cout << client << ' '
 		 << "-" << ' '
-		 << "-" << ' '
+		 << username << ' '
 		 << start_local << ' '
 		 << '"' << req.method << ' ' << req.uri << ' '
 				<< "HTTP/" << req.http_version_major << '.' << req.http_version_minor << "\" "
