@@ -10,6 +10,7 @@
 #include <boost/iostreams/device/back_inserter.hpp>
 #include <boost/iostreams/filtering_stream.hpp>
 #include <boost/format.hpp>
+#include <boost/algorithm/string.hpp>
 
 #include <iostream>
 
@@ -19,6 +20,7 @@
 
 using namespace std;
 namespace io = boost::iostreams;
+namespace ba = boost::algorithm;
 
 namespace zeep { namespace http {
 
@@ -31,7 +33,6 @@ struct status_string
 } kStatusStrings[] = {
 	{ cont,					"Continue" },
 	{ ok,					"OK" },
-	{ not_found,			"Not Found" },
 	{ created,				"Created" },
 	{ accepted,				"Accepted" },
 	{ no_content,			"No Content" },
@@ -41,26 +42,46 @@ struct status_string
 	{ not_modified,			"Not Modified" },
 	{ bad_request,			"Bad Request" },
 	{ unauthorized,			"Unauthorized" },
+	{ proxy_authentication_required,	"Proxy Authentication Required" },
 	{ forbidden,			"Forbidden" },
 	{ not_found,			"Not Found" },
+	{ method_not_allowed,	"Method not allowed" },
 	{ internal_server_error,"Internal Server Error" },
 	{ not_implemented,		"Not Implemented" },
 	{ bad_gateway,			"Bad Gateway" },
 	{ service_unavailable,	"Service Unavailable" }
+}, kStatusDescriptions[] = {
+	{ moved_permanently,	"The document requested was moved permanently to a new location" },
+	{ moved_temporarily,	"The document requested was moved temporarily to a new location" },
+	{ not_modified,			"The requested document was not modified" },
+	{ bad_request,			"There was an error in the request, e.g. an incorrect method or a malformed URI" },
+	{ unauthorized,			"You are not authorized to access this location" },
+	{ proxy_authentication_required,	"You are not authorized to use this proxy" },
+	{ forbidden,			"Access to this location is forbidden" },
+	{ not_found,			"The requested web page was not found on this server." },
+	{ internal_server_error,"An internal error prevented the server from processing your request" },
+	{ not_implemented,		"Your request could not be handled since the required code is not implemented" },
+	{ bad_gateway,			"The server, while acting as a gateway or proxy, received an invalid response from the upstream server it accessed in attempting to fulfill the request. " },
+	{ service_unavailable,	"The service is unavailable at this moment, try again later" }
 };
 
 const int
 	kStatusStringCount = sizeof(kStatusStrings) / sizeof(status_string);
+
+const int
+	kStatusDescriptionCount = sizeof(kStatusDescriptions) / sizeof(status_string);
 	
+}
+
 string get_status_text(status_type status)
 {
 	string result = "Internal Service Error";
 	
-	for (int i = 0; i < kStatusStringCount; ++i)
+	for (int i = 0; i < detail::kStatusStringCount; ++i)
 	{
-		if (kStatusStrings[i].code == status)
+		if (detail::kStatusStrings[i].code == status)
 		{
-			result = kStatusStrings[i].text;
+			result = detail::kStatusStrings[i].text;
 			break;
 		}
 	}
@@ -68,15 +89,30 @@ string get_status_text(status_type status)
 	return result;
 }
 	
+string get_status_description(status_type status)
+{
+	string result = "An internal error prevented the server from processing your request";
+	
+	for (int i = 0; i < detail::kStatusDescriptionCount; ++i)
+	{
+		if (detail::kStatusDescriptions[i].code == status)
+		{
+			result = detail::kStatusDescriptions[i].text;
+			break;
+		}
+	}
+	
+	return result;
 }
-
+	
 // ----------------------------------------------------------------------------
 
 namespace
 {
 const char
 		kNameValueSeparator[] = { ':', ' ' },
-		kCRLF[] = { '\r', '\n' };
+		kCRLF[] = { '\r', '\n' },
+		kZERO[] = { '0' };
 }
 
 reply::reply(int version_major, int version_minor)
@@ -116,6 +152,18 @@ reply::~reply()
 	delete m_data;
 }
 
+void reply::clear()
+{
+	delete m_data;
+	m_data = nullptr;
+	
+	m_status = ok;
+	m_status_line = "OK";
+	m_headers.clear();
+	m_buffer.clear();
+	m_content.clear();
+}
+
 reply& reply::operator=(const reply& rhs)
 {
 	if (this != &rhs)
@@ -157,6 +205,22 @@ void reply::set_header(const string& name, const string& value)
 	}
 }
 
+bool reply::keep_alive() const
+{
+	bool result = false;
+	
+	foreach (const header& h, m_headers)
+	{
+		if (ba::iequals(h.name, "Connection") and ba::iequals(h.value, "keep-alive"))
+		{
+			result = true;
+			break;
+		}
+	}
+	
+	return result;
+}
+
 void reply::set_content(xml::element* data)
 {
 	xml::document doc;
@@ -223,7 +287,7 @@ string reply::get_content_type() const
 	
 	foreach (const header& h, m_headers)
 	{
-		if (h.name == "Content-Type")
+		if (ba::iequals(h.name, "Content-Type"))
 		{
 			result = h.value;
 			break;
@@ -238,7 +302,7 @@ void reply::set_content_type(
 {
 	foreach (header& h, m_headers)
 	{
-		if (h.name == "Content-Type")
+		if (ba::iequals(h.name, "Content-Type"))
 		{
 			h.value = type;
 			break;
@@ -249,7 +313,7 @@ void reply::set_content_type(
 void reply::to_buffers(vector<boost::asio::const_buffer>& buffers)
 {
 	m_status_line = (boost::format("HTTP/%1%.%2% %3% %4%\r\n")
-		% m_version_major % m_version_minor % m_status % detail::get_status_text(m_status)).str();
+		% m_version_major % m_version_minor % m_status % get_status_text(m_status)).str();
 	buffers.push_back(boost::asio::buffer(m_status_line));
 	
 	foreach (header& h, m_headers)
@@ -279,14 +343,14 @@ bool reply::data_to_buffers(vector<boost::asio::const_buffer>& buffers)
 		else if (m_buffer.size() > kMaxChunkSize)
 			m_buffer.erase(m_buffer.begin() + kMaxChunkSize, m_buffer.end());
 		
-		streamsize n = m_data->readsome(&m_buffer[0], m_buffer.size());
+		streamsize n = m_data->rdbuf()->sgetn(&m_buffer[0], m_buffer.size());
 
 		// chunked encoding?
 		if (m_version_major > 1 or m_version_minor >= 1)
 		{
 			if (n == 0)
 			{
-				buffers.push_back(boost::asio::buffer("0"));
+				buffers.push_back(boost::asio::buffer(kZERO));
 				buffers.push_back(boost::asio::buffer(kCRLF));
 				buffers.push_back(boost::asio::buffer(kCRLF));
 				delete m_data;
@@ -323,7 +387,7 @@ string reply::get_as_text()
 {
 	// for best performance, we pre calculate memory requirements and reserve that first
 	m_status_line = (boost::format("HTTP/%1%.%2% %3% %4%\r\n")
-		% m_version_major % m_version_minor % m_status % detail::get_status_text(m_status)).str();
+		% m_version_major % m_version_minor % m_status % get_status_text(m_status)).str();
 
 	string result;
 	result.reserve(get_size());
@@ -353,14 +417,43 @@ size_t reply::get_size() const
 	return size;
 }
 
-reply reply::stock_reply(status_type status)
+reply reply::stock_reply(status_type status, const string& info)
 {
 	reply result;
 
 	if (status != not_modified)
 	{
 		stringstream text;
-		text << "<html><body><h1>" << detail::get_status_text(status) << "</h1></body></html>";
+
+		text << "<html>" << endl
+			 << "  <body>" << endl
+			 << "    <h1>" << get_status_text(status) << "</h1>" << endl;
+		
+		if (not info.empty())
+		{
+			text << "    <p>";
+		
+			foreach (char c, info)
+			{
+				switch (c)
+				{
+					case '&':	text << "&amp;"; break;
+					case '<':	text << "&lt;"; break;
+					case '>':	text << "&gt;"; break;
+					case 0:		break;	// silently ignore
+					default:	if ((c >= 1 and c <= 8) or (c >= 0x0b and c <= 0x0c) or (c >= 0x0e and c <= 0x1f) or c == 0x7f)
+									text << "&#" << hex << c << ';';
+								else	
+									text << c;
+								break;
+				}
+			}
+			
+			text << "</p>" << endl;
+		}
+		
+		text << "  </body>" << endl
+			 << "</html>";
 		result.set_content(text.str(), "text/html; charset=utf-8");
 	}
 
@@ -369,13 +462,18 @@ reply reply::stock_reply(status_type status)
 	return result;
 }
 
+reply reply::stock_reply(status_type status)
+{
+	return stock_reply(status, "");
+}
+
 reply reply::redirect(const std::string& location)
 {
 	reply result;
 
 	result.m_status = moved_temporarily;
 
-	string text = detail::get_status_text(moved_temporarily);
+	string text = get_status_text(moved_temporarily);
 	result.m_content =
 		string("<html><head><title>") + text + "</title></head><body><h1>" +
  		boost::lexical_cast<string>(moved_temporarily) + ' ' + text + "</h1></body></html>";
@@ -387,5 +485,25 @@ reply reply::redirect(const std::string& location)
 	return result;
 }
 
+void reply::debug(ostream& os) const
+{
+	os << (boost::format("HTTP/%1%.%2% %3% %4%")
+		% m_version_major % m_version_minor % m_status % get_status_text(m_status)) << endl;
+	foreach (const header& h, m_headers)
+		os << h.name << ": " << h.value << endl;
+}
+
+ostream& operator<<(ostream& lhs, reply& rhs)
+{
+	vector<boost::asio::const_buffer> buffers;
+
+	rhs.to_buffers(buffers);
+
+	foreach(auto& b, buffers)
+		lhs.write(boost::asio::buffer_cast<const char*>(b), boost::asio::buffer_size(b));
+
+	return lhs;
+}
+
 }
 }
