@@ -11,10 +11,16 @@
 
 #include <iomanip>
 
+#if defined(_MSC_VER) /*<= 1600*/
+#include <boost/tr1/cmath.hpp>
+using namespace std::tr1;
+#else
+#include <cmath>
+#endif
+
 #include <boost/foreach.hpp>
 #define foreach BOOST_FOREACH
 #include <boost/algorithm/string.hpp>
-#include <boost/tr1/cmath.hpp>
 
 #include <zeep/http/webapp.hpp>
 #include <zeep/http/webapp/el.hpp>
@@ -121,10 +127,18 @@ class float_object_impl : public detail::object_impl
 	virtual void	print(ostream& os) const		{ os << m_v; }
 	virtual int		compare(object_impl* rhs) const;
 
-	virtual int64	to_int() const						{ return static_cast<int64>(tr1::round(m_v)); }
+	virtual int64	to_int() const						{ return static_cast<int64>(round(m_v)); }
 	virtual double	to_double() const					{ return m_v; }
 	virtual string	to_str() const						{ return boost::lexical_cast<string>(m_v); }
-	virtual string	to_JSON() const						{ return boost::lexical_cast<string>(m_v); }
+	virtual string	to_JSON() const
+	{
+		if (std::isnan(m_v))
+			return "\"NaN\"";
+		else if (std::isinf(m_v))
+			return "\"Infinite\"";
+		else
+			return boost::lexical_cast<string>(m_v);
+	}
 	
 	double			m_v;
 };
@@ -208,26 +222,44 @@ class vector_object_impl : public detail::base_array_object_impl
 							m_v.push_back(object(s));
 					}
 
+					vector_object_impl(const vector<float>& v)
+					{
+						foreach (float f, v)
+							m_v.push_back(object(f));
+					}
+
 	virtual object&	at(uint32 ix)
 					{
+						if (ix >= m_v.size())
+							throw logic_error("index out of bounds");
 						return m_v[ix];
 					}
 					
 	virtual const object
 					at(uint32 ix) const
 					{
-						return m_v[ix];
+						object result;
+						if (ix < m_v.size())
+							result = m_v[ix];
+						return result;
 					}
 
 	virtual object&	operator[](const object& index)
 					{
-						return m_v[index.as<uint32>()];
+						uint32 ix = index.as<uint32>();
+						if (ix >= m_v.size())
+							throw logic_error("index out of bounds");
+						return m_v[ix];
 					}
 
 	virtual const object
 					operator[](const object& index) const
 					{
-						return m_v[index.as<uint32>()];
+						object result;
+						uint32 ix = index.as<uint32>();
+						if (ix < m_v.size())
+							result = m_v[ix];
+						return result;
 					}
 
 	virtual void	print(ostream& os) const
@@ -519,6 +551,11 @@ object::object(const vector<string>& v)
 {
 }
 
+object::object(const vector<float>& v)
+	: m_impl(new vector_object_impl(v))
+{
+}
+
 object::object(bool v)
 //	: m_impl(new bool_object_impl(v))
 	: m_impl(new int_object_impl(v))
@@ -1564,7 +1601,7 @@ object interpreter::parse_primary_expr()
 					if (index.empty() or (result.type() != object::array_type and result.type() != object::struct_type))
 						result = object();
 					else
-						result = const_cast<object&>(result)[index];
+						result = const_cast<const object&>(result)[index];
 					continue;
 				}
 
