@@ -8,57 +8,157 @@
 
 #include <zeep/config.hpp>
 
+#include <map>
+#include <set>
+
 #include <boost/foreach.hpp>
 #define foreach BOOST_FOREACH
 #include <boost/algorithm/string.hpp>
+#include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/date_time/local_time/local_time.hpp>
 #include <boost/iostreams/copy.hpp>
 #include <boost/filesystem/fstream.hpp>
+#include <boost/random/random_device.hpp>
 
 #include <zeep/http/webapp.hpp>
 #include <zeep/xml/unicode_support.hpp>
 #include <zeep/http/webapp/el.hpp>
+#include <zeep/http/md5.hpp>
 
 using namespace std;
 namespace ba = boost::algorithm;
 namespace io = boost::iostreams;
 namespace fs = boost::filesystem;
+namespace pt = boost::posix_time;
 
 namespace zeep { namespace http {
 
 // --------------------------------------------------------------------
 //
 
-webapp::webapp(
-	const std::string&	ns,
-	const fs::path&		docroot)
+// add a name/value pair as a string formatted as 'name=value'
+void parameter_map::add(const string& param)
+{
+	string name, value;
+	
+	string::size_type d = param.find('=');
+	if (d != string::npos)
+	{
+		name = param.substr(0, d);
+		value = param.substr(d + 1);
+	}
+	
+	add(name, value);
+}
+
+void parameter_map::add(string name, string value)
+{
+	name = decode_url(name);
+	if (not value.empty())
+		value = decode_url(value);
+	
+	insert(make_pair(name, parameter_value(value, false)));
+}
+
+void parameter_map::replace(string name, string value)
+{
+	if (count(name))
+		erase(lower_bound(name), upper_bound(name));
+	add(name, value);
+}
+
+// --------------------------------------------------------------------
+//
+
+struct auth_info
+{
+	auth_info(const string& realm);
+	
+	bool validate(const string& method, const string& uri, const string& ha1, map<string,string>& info);
+
+	string get_challenge() const;
+	bool stale() const;
+	
+	string m_nonce, m_realm;
+	set<uint32> m_replay_check;
+	pt::ptime m_created;
+};
+
+auth_info::auth_info(const string& realm)
+	: m_realm(realm)
+{
+	using namespace boost::gregorian;
+
+	boost::random::random_device rng;
+	uint32 data[4] = { rng(), rng(), rng(), rng() };
+
+	m_nonce = md5(data, sizeof(data)).finalise();
+	m_created = pt::second_clock::local_time();
+}
+
+string auth_info::get_challenge() const
+{
+	string challenge = "Digest ";
+	challenge += "realm=\"" + m_realm + "\", qop=\"auth\", nonce=\"" + m_nonce + '"';
+	return challenge;
+}
+
+bool auth_info::stale() const
+{
+	pt::time_duration age = pt::second_clock::local_time() - m_created;
+	return age.total_seconds() > 1800;
+}
+
+bool auth_info::validate(const string& method, const string& uri, const string& ha1, map<string,string>& info)
+{
+	bool valid = false;
+	
+	uint32 nc = strtol(info["nc"].c_str(), nullptr, 16);
+	if (not m_replay_check.count(nc))
+	{
+		string ha2 = md5(method + ':' + info["uri"]).finalise();
+		
+		string response = md5(
+			ha1 + ':' +
+			info["nonce"] + ':' +
+			info["nc"] + ':' +
+			info["cnonce"] + ':' +
+			info["qop"] + ':' +
+			ha2).finalise();
+		
+		valid = info["response"] == response;
+
+		// keep a list of seen nc-values in m_replay_check
+		m_replay_check.insert(nc);
+	}
+	return valid;
+}
+
+// --------------------------------------------------------------------
+//
+
+basic_webapp::basic_webapp(const string& ns, const fs::path& docroot)
 	: m_ns(ns)
 	, m_docroot(docroot)
 {
-	m_processor_table["include"] =	boost::bind(&webapp::process_include,	this, _1, _2, _3);
-	m_processor_table["if"] =		boost::bind(&webapp::process_if,		this, _1, _2, _3);
-	m_processor_table["iterate"] =	boost::bind(&webapp::process_iterate,	this, _1, _2, _3);
-	m_processor_table["for"] =		boost::bind(&webapp::process_for,		this, _1, _2, _3);
-	m_processor_table["number"] =	boost::bind(&webapp::process_number,	this, _1, _2, _3);
-	m_processor_table["options"] =	boost::bind(&webapp::process_options,	this, _1, _2, _3);
-	m_processor_table["option"] =	boost::bind(&webapp::process_option,	this, _1, _2, _3);
-	m_processor_table["checkbox"] =	boost::bind(&webapp::process_checkbox,	this, _1, _2, _3);
-	m_processor_table["url"] =		boost::bind(&webapp::process_url, 		this, _1, _2, _3);
-	m_processor_table["param"] =	boost::bind(&webapp::process_param,		this, _1, _2, _3);
-	m_processor_table["embed"] =	boost::bind(&webapp::process_embed,		this, _1, _2, _3);
+	m_processor_table["include"] =	boost::bind(&basic_webapp::process_include,	this, _1, _2, _3);
+	m_processor_table["if"] =		boost::bind(&basic_webapp::process_if,		this, _1, _2, _3);
+	m_processor_table["iterate"] =	boost::bind(&basic_webapp::process_iterate,	this, _1, _2, _3);
+	m_processor_table["for"] =		boost::bind(&basic_webapp::process_for,		this, _1, _2, _3);
+	m_processor_table["number"] =	boost::bind(&basic_webapp::process_number,	this, _1, _2, _3);
+	m_processor_table["options"] =	boost::bind(&basic_webapp::process_options,	this, _1, _2, _3);
+	m_processor_table["option"] =	boost::bind(&basic_webapp::process_option,	this, _1, _2, _3);
+	m_processor_table["checkbox"] =	boost::bind(&basic_webapp::process_checkbox,this, _1, _2, _3);
+	m_processor_table["url"] =		boost::bind(&basic_webapp::process_url, 	this, _1, _2, _3);
+	m_processor_table["param"] =	boost::bind(&basic_webapp::process_param,	this, _1, _2, _3);
+	m_processor_table["embed"] =	boost::bind(&basic_webapp::process_embed,	this, _1, _2, _3);
 }
 
-webapp::~webapp()
+basic_webapp::~basic_webapp()
 {
 }
 
-void webapp::set_docroot(
-	const fs::path&		path)
-{
-	m_docroot = path;
-}
-
-void webapp::handle_request(
+void basic_webapp::handle_request(
 	const request&		req,
 	reply&				rep)
 {
@@ -68,7 +168,7 @@ void webapp::handle_request(
 	if (req.method != "GET" and req.method != "POST" and req.method != "PUT" and
 		req.method != "OPTIONS" and req.method != "HEAD")
 	{
-		rep = reply::stock_reply(bad_request);
+		create_error_reply(req, bad_request, rep);
 		return;
 	}
 
@@ -114,9 +214,14 @@ void webapp::handle_request(
 		scope.put("baseuri", uri);
 		scope.put("mobile", req.is_mobile());
 
-		handler_map::iterator handler = m_dispatch_table.find(uri);
+		auto handler = find_if(m_dispatch_table.begin(), m_dispatch_table.end(),
+			[uri](const mount_point& m) -> bool { return m.path == uri; });
+		
 		if (handler == m_dispatch_table.end())
-			handler = m_dispatch_table.find(action);
+		{
+			handler = find_if(m_dispatch_table.begin(), m_dispatch_table.end(),
+				[action](const mount_point& m) -> bool { return m.path == action; });
+		}
 
 		if (handler != m_dispatch_table.end())
 		{
@@ -128,9 +233,20 @@ void webapp::handle_request(
 			}
 			else
 			{
+				// Do authentication here, if needed
+				if (not handler->realm.empty())
+				{
+// #pragma message("This sucks, please fix")
+					// this is extremely dirty...
+					auto& r = const_cast<request&>(req);
+					validate_authentication(r, handler->realm);
+
+					scope.put("username", req.username);
+				}
+				
 				init_scope(scope);
 				
-				handler->second(req, scope, rep);
+				handler->handler(req, scope, rep);
 				
 				if (req.method == "HEAD")
 					rep.set_content("", rep.get_content_type());
@@ -141,32 +257,87 @@ void webapp::handle_request(
 	}
 	catch (unauthorized_exception& e)
 	{
-		create_unauth_reply(e.m_stale, e.m_realm, rep);
+		create_unauth_reply(req, e.m_stale, e.m_realm, rep);
 	}
 	catch (status_type& s)
 	{
-		rep = reply::stock_reply(s);
+		create_error_reply(req, s, rep);
 	}
 	catch (std::exception& e)
 	{
-		el::scope scope(req);
-		scope.put("errormsg", el::object(e.what()));
-
-		create_reply_from_template("error.html", scope, rep);
+		create_error_reply(req, internal_server_error, e.what(), rep);
 	}
 }
 
-void webapp::create_unauth_reply(bool stale, const string& realm, reply& rep)
+void basic_webapp::create_unauth_reply(const request& req, bool stale, const string& realm, const string& authenticate, reply& rep)
 {
-	rep = reply::stock_reply(unauthorized);
+	boost::unique_lock<boost::mutex> lock(m_auth_mutex);
+	
+	create_error_reply(req, unauthorized, get_status_text(unauthorized), rep);
+	
+	m_auth_info.push_back(auth_info(realm));
+
+	string challenge = m_auth_info.back().get_challenge();
+	if (stale)
+		challenge += ", stale=\"true\"";
+
+	rep.set_header(authenticate, challenge);
 }
 
-void webapp::mount(const std::string& path, handler_type handler)
+void basic_webapp::create_error_reply(const request& req, status_type status, reply& rep)
 {
-	m_dispatch_table[path] = handler;
+	create_error_reply(req, status, "", rep);
 }
 
-void webapp::handle_file(
+void basic_webapp::create_error_reply(const request& req, status_type status, const string& message, reply& rep)
+{
+	el::scope scope(req);
+	
+	el::object error;
+	error["nr"] = static_cast<int>(status);
+	error["head"] = get_status_text(status);
+	error["description"] = get_status_description(status);
+	
+	if (not message.empty())
+		error["message"] = message;
+	
+	el::object request;
+	request["line"] = 
+		ba::starts_with(req.uri, "http://") ? 
+			(boost::format("%1% %2% HTTP%3%/%4%") % req.method % req.uri % req.http_version_major % req.http_version_minor).str() :
+			(boost::format("%1% http://%2%%3% HTTP%4%/%5%") % req.method % req.get_header("Host") % req.uri % req.http_version_major % req.http_version_minor).str();
+	request["username"] = req.username;
+	error["request"] = request;
+	
+	scope.put("error", error);
+
+	create_reply_from_template("error.html", scope, rep);
+	rep.set_status(status);
+}
+
+void basic_webapp::mount(const std::string& path, handler_type handler)
+{
+	mount(path, "", handler);
+}
+
+void basic_webapp::mount(const std::string& path, const std::string& realm, handler_type handler)
+{
+	auto mp = find_if(m_dispatch_table.begin(), m_dispatch_table.end(), [path](const mount_point& mp) -> bool { return mp.path == path; });
+	if (mp == m_dispatch_table.end())
+	{
+		mount_point p = { path, realm, handler };
+		m_dispatch_table.push_back(p);
+	}
+	else
+	{
+		if (mp->realm != realm)
+			throw logic_error("realms not equal");
+
+		mp->handler = handler;
+	}
+}
+
+void basic_webapp::handle_file(
 	const zeep::http::request&	request,
 	const el::scope&			scope,
 	zeep::http::reply&			reply)
@@ -175,11 +346,16 @@ void webapp::handle_file(
 	using namespace boost::posix_time;
 	
 	fs::path file = get_docroot() / scope["baseuri"].as<string>();
+	if (not fs::exists(file))
+	{
+		reply = zeep::http::reply::stock_reply(not_found);
+		return;
+	}	
 
 	string ifModifiedSince;
 	foreach (const zeep::http::header& h, request.headers)
 	{
-		if (h.name == "If-Modified-Since")
+		if (ba::iequals(h.name, "If-Modified-Since"))
 		{
 			local_date_time modifiedSince(local_sec_clock::local_time(time_zone_ptr()));
 
@@ -239,9 +415,33 @@ void webapp::handle_file(
 	reply.set_header("Last-Modified", s.str());
 }
 
-void webapp::load_template(
-	const std::string&	file,
-	xml::document&		doc)
+void basic_webapp::get_cookies(
+	const el::scope&	scope,
+	parameter_map&		cookies)
+{
+	const request& req = scope.get_request();
+	foreach (const header& h, req.headers)
+	{
+		if (h.name != "Cookie")
+			continue;
+		
+		vector<string> rawCookies;
+		ba::split(rawCookies, h.value, ba::is_any_of(";"));
+		
+		foreach (string& cookie, rawCookies)
+		{
+			ba::trim(cookie);
+			cookies.add(cookie);
+		}
+	}
+}
+
+void basic_webapp::set_docroot(const fs::path& path)
+{
+	m_docroot = path;
+}
+
+void basic_webapp::load_template(const string& file, xml::document& doc)
 {
 	fs::ifstream data(m_docroot / file, ios::binary);
 	if (not data.is_open())
@@ -256,7 +456,7 @@ void webapp::load_template(
 		    DWORD dw = ::GetLastError();
 			if (dw != NO_ERROR)
 			{
-			    char* lpMsgBuf;
+			    char* lpMsgBuf = nullptr;
 				int m = ::FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
 					NULL, dw, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPSTR)&lpMsgBuf, 0, NULL);
 			
@@ -282,10 +482,7 @@ void webapp::load_template(
 	doc.read(data);
 }
 
-void webapp::create_reply_from_template(
-	const std::string&	file,
-	const el::scope&	scope,
-	reply&				reply)
+void basic_webapp::create_reply_from_template(const string& file, const el::scope& scope, reply& reply)
 {
 	xml::document doc;
 	doc.set_preserve_cdata(true);
@@ -297,10 +494,7 @@ void webapp::create_reply_from_template(
 	reply.set_content(doc);
 }
 
-void webapp::process_xml(
-	xml::node*			node,
-	const el::scope&	scope,
-	fs::path			dir)
+void basic_webapp::process_xml(xml::node* node, const el::scope& scope, fs::path dir)
 {
 	xml::text* text = dynamic_cast<xml::text*>(node);
 	
@@ -331,7 +525,7 @@ void webapp::process_xml(
 			else
 				throw exception((boost::format("unimplemented <mrs:%1%> tag") % e->name()).str());
 		}
-		catch (std::exception& ex)
+		catch (exception& ex)
 		{
 			xml::node* replacement = new xml::text(
 				(boost::format("Error processing directive 'mrs:%1%': %2%") %
@@ -373,17 +567,12 @@ void webapp::process_xml(
 	}
 }
 
-void webapp::add_processor(
-	const std::string&	name,
-	processor_type		processor)
+void basic_webapp::add_processor(const string& name, processor_type processor)
 {
 	m_processor_table[name] = processor;
 }
 
-void webapp::process_include(
-	xml::element*		node,
-	const el::scope&	scope,
-	fs::path			dir)
+void basic_webapp::process_include(xml::element* node, const el::scope& scope, fs::path dir)
 {
 	// an include directive, load file and include resulting content
 	string file = node->get_attribute("file");
@@ -405,10 +594,7 @@ void webapp::process_include(
 	process_xml(replacement, scope, (dir / file).parent_path());
 }
 
-void webapp::process_if(
-	xml::element*		node,
-	const el::scope&	scope,
-	fs::path			dir)
+void basic_webapp::process_if(xml::element* node, const el::scope& scope, fs::path dir)
 {
 	string test = node->get_attribute("test");
 	if (evaluate_el(scope, test))
@@ -426,10 +612,7 @@ void webapp::process_if(
 	}
 }
 
-void webapp::process_iterate(
-	xml::element*		node,
-	const el::scope&	scope,
-	fs::path			dir)
+void basic_webapp::process_iterate(xml::element* node, const el::scope& scope, fs::path dir)
 {
 	el::object collection = scope[node->get_attribute("collection")];
 	if (collection.type() != el::object::array_type)
@@ -457,10 +640,7 @@ void webapp::process_iterate(
 	}
 }
 
-void webapp::process_for(
-	xml::element*		node,
-	const el::scope&	scope,
-	fs::path			dir)
+void basic_webapp::process_for(xml::element* node, const el::scope& scope, fs::path dir)
 {
 	el::object b, e;
 	
@@ -496,10 +676,7 @@ class with_thousands : public numpunct<char>
 //	char_type do_decimal_point() const	{ return dsp; }
 };
 
-void webapp::process_number(
-	xml::element*		node,
-	const el::scope&	scope,
-	fs::path			dir)
+void basic_webapp::process_number(xml::element* node, const el::scope& scope, fs::path dir)
 {
 	string number = node->get_attribute("n");
 	string format = node->get_attribute("f");
@@ -550,10 +727,7 @@ void webapp::process_number(
 	parent->insert(node, replacement);
 }
 
-void webapp::process_options(
-	xml::element*		node,
-	const el::scope&	scope,
-	fs::path			dir)
+void basic_webapp::process_options(xml::element* node, const el::scope& scope, fs::path dir)
 {
 	el::object collection = scope[node->get_attribute("collection")];
 	if (collection.type() != el::object::array_type)
@@ -595,10 +769,7 @@ void webapp::process_options(
 	}
 }
 
-void webapp::process_option(
-	xml::element*		node,
-	const el::scope&	scope,
-	fs::path			dir)
+void basic_webapp::process_option(xml::element* node, const el::scope& scope, fs::path dir)
 {
 	string value = node->get_attribute("value");
 	if (not value.empty())
@@ -634,10 +805,7 @@ void webapp::process_option(
 	}
 }
 
-void webapp::process_checkbox(
-	xml::element*		node,
-	const el::scope&	scope,
-	fs::path			dir)
+void basic_webapp::process_checkbox(xml::element* node, const el::scope& scope, fs::path dir)
 {
 	string name = node->get_attribute("name");
 	if (not name.empty())
@@ -674,10 +842,7 @@ void webapp::process_checkbox(
 	}
 }
 
-void webapp::process_url(
-	xml::element*		node,
-	const el::scope&	scope,
-	fs::path			dir)
+void basic_webapp::process_url(xml::element* node, const el::scope& scope, fs::path dir)
 {
 	string var = node->get_attribute("var");
 	
@@ -714,18 +879,12 @@ void webapp::process_url(
 	s.put(var, url);
 }
 
-void webapp::process_param(
-	xml::element*		node,
-	const el::scope&	scope,
-	fs::path			dir)
+void basic_webapp::process_param(xml::element* node, const el::scope& scope, fs::path dir)
 {
 	throw exception("Invalid XML, cannot have a stand-alone mrs:param element");
 }
 
-void webapp::process_embed(
-	xml::element*		node,
-	const el::scope&	scope,
-	fs::path			dir)
+void basic_webapp::process_embed(xml::element* node, const el::scope& scope, fs::path dir)
 {
 	// an embed directive, load xml from attribute and include parsed content
 	string xml = scope[node->get_attribute("var")].as<string>();
@@ -746,42 +905,27 @@ void webapp::process_embed(
 	process_xml(replacement, scope, dir);
 }
 
-void webapp::init_scope(
-	el::scope&			scope)
+void basic_webapp::init_scope(el::scope& scope)
 {
 }		
 
-void webapp::get_cookies(
-	const el::scope&	scope,
-	parameter_map&		cookies)
-{
-	const request& req = scope.get_request();
-	foreach (const header& h, req.headers)
-	{
-		if (h.name != "Cookie")
-			continue;
-		
-		vector<string> rawCookies;
-		ba::split(rawCookies, h.value, ba::is_any_of(";"));
-		
-		foreach (string& cookie, rawCookies)
-		{
-			ba::trim(cookie);
-			cookies.add(cookie);
-		}
-	}
-}
-
-void webapp::get_parameters(
-	const el::scope&	scope,
-	parameter_map&		parameters)
+void basic_webapp::get_parameters(const el::scope& scope, parameter_map& parameters)
 {
 	const request& req = scope.get_request();
 	
 	string ps;
 	
 	if (req.method == "POST")
-		ps = req.payload;
+	{
+		string contentType = req.get_header("Content-Type");
+		
+		if (ba::starts_with(contentType, "application/x-www-form-urlencoded"))
+			ps = req.payload;
+//		else
+//		{
+//			
+//		}
+	}
 	else if (req.method == "GET" or req.method == "PUT")
 	{
 		string::size_type d = req.uri.find('?');
@@ -810,40 +954,75 @@ void webapp::get_parameters(
 // --------------------------------------------------------------------
 //
 
-// add a name/value pair as a string formatted as 'name=value'
-void parameter_map::add(
-	const string&		param)
+string basic_webapp::validate_authentication(const string& authorization,
+	const string& method, const string& uri, const string& realm)
 {
-	string name, value;
+	if (authorization.empty())
+		throw unauthorized_exception(false, realm);
+
+	// That was easy, now check the response
 	
-	string::size_type d = param.find('=');
-	if (d != string::npos)
+	map<string,string> info;
+	
+	boost::regex re("(\\w+)=(?|\"([^\"]*)\"|'([^']*)'|(\\w+))(?:,\\s*)?");
+	const char* b = authorization.c_str();
+	const char* e = b + authorization.length();
+	boost::match_results<const char*> m;
+	while (b < e and boost::regex_search(b, e, m, re))
 	{
-		name = param.substr(0, d);
-		value = param.substr(d + 1);
+		info[string(m[1].first, m[1].second)] = string(m[2].first, m[2].second);
+		b = m[0].second;
 	}
 	
-	add(name, value);
+	if (info["realm"] != realm)
+		throw unauthorized_exception(false, realm);
+
+	string ha1 = get_hashed_password(info["username"], realm);
+
+	// lock to avoid accessing m_auth_info from multiple threads at once
+	boost::unique_lock<boost::mutex> lock(m_auth_mutex);
+	bool authorized = false, stale = false;
+
+	for (auto auth = m_auth_info.begin(); auth != m_auth_info.end(); ++auth)
+	{
+		if (auth->m_realm == realm and auth->m_nonce == info["nonce"]
+			and auth->validate(method, uri, ha1, info))
+		{
+			authorized = true;
+			stale = auth->stale();
+			if (stale)
+				m_auth_info.erase(auth);
+			break;
+		}
+	}
+
+	if (stale or not authorized)
+		throw unauthorized_exception(stale, realm);
+
+	return info["username"];
 }
 
-void parameter_map::add(
-	string		name,
-	string		value)
+string basic_webapp::get_hashed_password(const string& username, const string& realm)
 {
-	name = decode_url(name);
-	if (not value.empty())
-		value = decode_url(value);
-	
-	insert(make_pair(name, parameter_value(value, false)));
+	return "";
 }
 
-void parameter_map::replace(
-	string			name,
-	string			value)
+// --------------------------------------------------------------------
+//
+
+webapp::webapp(const std::string& ns, const boost::filesystem::path& docroot)
+	: basic_webapp(ns, docroot)
 {
-	if (count(name))
-		erase(lower_bound(name), upper_bound(name));
-	add(name, value);
+}
+
+webapp::~webapp()
+{
+}
+
+void webapp::handle_request(const request& req, reply& rep)
+{
+	server::log() << req.uri;
+	basic_webapp::handle_request(req, rep);
 }
 
 }
