@@ -12,14 +12,12 @@
 #include <cmath>
 #include <map>
 
-#include <boost/tr1/cmath.hpp>
 #include <boost/function.hpp>
 #include <boost/bind.hpp>
 #include <boost/foreach.hpp>
 #define foreach BOOST_FOREACH
 #include <boost/algorithm/string.hpp>
 #include <boost/lexical_cast.hpp>
-#include <boost/shared_ptr.hpp>
 
 #include <zeep/exception.hpp>
 #include <zeep/xml/node.hpp>
@@ -31,7 +29,6 @@
 //extern int VERBOSE;
 
 using namespace std;
-using namespace tr1;
 namespace ba = boost::algorithm;
 
 namespace zeep { namespace xml {
@@ -308,7 +305,7 @@ const bool object::as<bool>() const
 	bool result;
 	switch (m_type)
 	{
-		case ot_number:		result = m_number != 0 and not tr1::isnan(m_number); break;
+		case ot_number:		result = m_number != 0 and not std::isnan(m_number); break;
 		case ot_node_set:	result = not m_node_set.empty(); break;
 		case ot_string:		result = not m_string.empty(); break;
 		case ot_boolean:	result = m_boolean; break;
@@ -338,7 +335,7 @@ const int object::as<int>() const
 {
 	if (m_type != ot_number)
 		throw exception("object is not of type number");
-	return static_cast<int>(tr1::round(m_number));
+	return static_cast<int>(round(m_number));
 }
 
 template<>
@@ -641,8 +638,14 @@ class expression
 	virtual void		print(int level) = 0;
 };
 
-typedef boost::shared_ptr<expression>	expression_ptr;
-typedef list<expression_ptr>			expression_list;
+typedef std::shared_ptr<expression>	expression_ptr;
+typedef list<expression_ptr>		expression_list;
+
+// needed for CLang/libc++ on FreeBSD 10
+expression* get_pointer(shared_ptr<expression> const & p)
+{
+	return p.get();
+}
 
 // --------------------------------------------------------------------
 
@@ -1547,7 +1550,7 @@ template<>
 object core_function_expression<cf_Round>::evaluate(expression_context& context)
 {
 	object v = m_args.front()->evaluate(context);
-	return tr1::round(v.as<double>());
+	return round(v.as<double>());
 }
 
 // --------------------------------------------------------------------
