@@ -149,7 +149,7 @@ struct struct_serializer<Archive,std::pair<T, U> >
 
 // now construct a server that can do several things:
 // ListDatabanks:	simply return the list of searchable databanks
-// Count:			a simple call taking two parameters and returning one 
+// Count:			a simple call taking two parameters and returning one
 // Find:			complex search routine taking several parameters and returning a complex type
 
 class my_server : public zeep::server
@@ -205,7 +205,7 @@ my_server::my_server(const string& param)
 	: zeep::server("http://mrs.cmbi.ru.nl/mrsws/search", "zeep")
 	, m_param(param)
 {
-	set_location("http://131.174.88.174:10333");
+//	set_location("http://131.174.88.174:10333");
 
 	using namespace WSSearchNS;
 
@@ -215,23 +215,23 @@ my_server::my_server(const string& param)
 		;
 
 	zeep::xml::struct_serializer_impl<Hit>::set_struct_name("Hit");
-	
+
 	// The next call is needed since FindResult is defined in another namespace
 	SOAP_XML_SET_STRUCT_NAME(FindResult);
 
 	const char* kListDatabanksParameterNames[] = {
 		"databank"
 	};
-	
+
 	register_action("ListDatabanks", this, &my_server::ListDatabanks, kListDatabanksParameterNames);
 
 	const char* kCountParameterNames[] = {
 		"db", "booleanquery", "response"
 	};
-	
+
 	register_action("Count", this, &my_server::Count, kCountParameterNames);
-	
-	// a new way of mapping enum values to strings. 
+
+	// a new way of mapping enum values to strings.
 	zeep::xml::enum_map<Algorithm>::instance("Algorithm").add_enum()
 		( "Vector", Vector )
 		( "Dice", Dice )
@@ -248,7 +248,7 @@ my_server::my_server(const string& param)
 		"alltermsrequired", "booleanfilter", "resultoffset", "maxresultcount",
 		"out"
 	};
-	
+
 	register_action("Find", this, &my_server::Find, kFindParameterNames);
 
 	const char* kDateTimeTestParameterNames[] = {
@@ -260,7 +260,7 @@ my_server::my_server(const string& param)
 	const char* kForceStopParameterNames[] = {
 		"out"
 	};
-	
+
 	register_action("ForceStop", this, &my_server::ForceStop, kForceStopParameterNames);
 
 	const char* kPairTestParameterNames[] = {
@@ -268,7 +268,7 @@ my_server::my_server(const string& param)
 	};
 
 	zeep::xml::struct_serializer_impl<pair<int,int> >::set_struct_name("pair_of_ints");
-	
+
 	register_action("PairTest", this, &my_server::PairTest, kPairTestParameterNames);
 }
 
@@ -314,7 +314,7 @@ void my_server::Find(
 	h.title = "bla bla bla";
 	h.score = 1.0f;
 	h.v_ptime = boost::posix_time::microsec_clock::universal_time();
-	
+
 	out.hits.push_back(h);
 
 	h.db = "sprot";
@@ -322,7 +322,7 @@ void my_server::Find(
 	h.title = "aap <&> noot mies";
 	h.score = 0.8f;
 	h.opt_text = "Hallóóów";
-	
+
 	out.hits.push_back(h);
 
 	h.db = "param";
@@ -330,7 +330,7 @@ void my_server::Find(
 	h.title = m_param;
 	h.score = 0.6f;
 	h.opt_text.reset();
-	
+
 	out.hits.push_back(h);
 }
 
@@ -360,18 +360,30 @@ int main(int argc, const char* argv[])
  	for (;;)
  	{
  		cout << "restarting server" << endl;
- 		
+
 	    sigset_t new_mask, old_mask;
 	    sigfillset(&new_mask);
 	    pthread_sigmask(SIG_BLOCK, &new_mask, &old_mask);
-	
-		zeep::http::preforked_server<my_server> server("bla bla");
-		boost::thread t(
-			boost::bind(&zeep::http::preforked_server<my_server>::run, &server, "131.174.88.174", 10333, 2));
-		server.start();
+
+		zeep::http::preforked_server server([=]() -> zeep::http::server*
+		{
+			try
+			{
+				return new my_server("bla bla");
+			}
+			catch (const exception& e)
+			{
+				cerr << "Failed to launch server: " << e.what() << endl;
+				exit(1);
+			}
+		});
 		
+		boost::thread t(
+			boost::bind(&zeep::http::preforked_server::run, &server, "0.0.0.0", 10333, 2));
+		server.start();
+
 	    pthread_sigmask(SIG_SETMASK, &old_mask, 0);
-	
+
 		// Wait for signal indicating time to shut down.
 		sigset_t wait_mask;
 		sigemptyset(&wait_mask);
@@ -384,23 +396,23 @@ int main(int argc, const char* argv[])
 		int sig = 0;
 		sigwait(&wait_mask, &sig);
 	    pthread_sigmask(SIG_SETMASK, &old_mask, 0);
-	
+
 		server.stop();
 		t.join();
-		
+
 		if (sig == SIGCHLD)
 		{
 			int status, pid;
 			pid = waitpid(-1, &status, WUNTRACED);
-	
+
 			if (pid != -1 and WIFSIGNALED(status))
 				cout << "child " << pid << " terminated by signal " << WTERMSIG(status) << endl;
 			continue;
 		}
-		
+
 		if (sig == SIGHUP)
 			continue;
-		
+
 		break;
  	}
 #elif defined(_MSC_VER)
@@ -432,7 +444,7 @@ int main(int argc, const char* argv[])
 		pthread_sigmask(SIG_BLOCK, &wait_mask, 0);
 		int sig = 0;
 		sigwait(&wait_mask, &sig);
-	
+
 		server.stop();
 		t.join();
 
