@@ -7,8 +7,8 @@
 #define SOAP_HTTP_REPLY_HPP
 
 #include <vector>
+#include <memory>
 
-#include <boost/tr1/memory.hpp>
 #include <boost/asio/buffer.hpp>
 
 #include <zeep/http/header.hpp>
@@ -33,21 +33,31 @@ enum status_type
     unauthorized =          401,
     forbidden =             403,
     not_found =             404,
+	method_not_allowed =	405,
+	proxy_authentication_required =	407,
     internal_server_error = 500,
     not_implemented =       501,
     bad_gateway =           502,
     service_unavailable =   503
 };
 
+/// Return the error string for the status_type
+std::string get_status_text(status_type status);
+
+/// Return the string describing the status_type in more detail
+std::string get_status_description(status_type status);
+
 class reply
 {
   public:
 	/// Create a reply, default is HTTP 1.0. Use 1.1 if you want to use keep alive e.g.
 
 						reply(int version_major = 1, int version_minor = 0);
-						reply(const reply&);
+						reply(const reply& rhs);
 						~reply();
 	reply&				operator=(const reply&);
+	
+	void				clear();
 
 	void				set_version(int version_major, int version_minor);
 
@@ -55,6 +65,9 @@ class reply
 	void				set_header(const std::string& name,
 							const std::string& value);
 
+	/// If the reply contains Connection: keep-alive
+	bool				keep_alive() const;
+
 	std::string			get_content_type() const;
 	void				set_content_type(
 							const std::string& type);	///< Set the Content-Type header
@@ -85,13 +98,21 @@ class reply
 
 	/// Create a standard reply based on a HTTP status code	
 	static reply		stock_reply(status_type inStatus);
-
+	static reply		stock_reply(status_type inStatus, const std::string& info);
+	
 	/// Create a standard redirect reply with the specified \a location
 	static reply		redirect(const std::string& location);
 	
+	void				set_status(status_type status)			{ m_status = status; }
 	status_type			get_status() const						{ return m_status; }
 
+	void				debug(std::ostream& os) const;
+
+	friend std::ostream& operator<<(std::ostream&, reply&);
+
   private:
+	friend class reply_parser;
+
 	int					m_version_major, m_version_minor;
 	status_type			m_status;
 	std::string			m_status_line;
@@ -101,6 +122,8 @@ class reply
 	std::vector<char>	m_buffer;
 };
 
+std::ostream& operator<<(std::ostream&, reply&);
+
 }
 }
 
